#define __cplusplus__
#ifndef __DATA__
#define __DATA__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

using namespace std;

/*! \class Data
 *  \brief A class representing a multidimensional dataset.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

 A dataset is a set of observations on a set of \b NbPts sites. These observations are supposed of dimension \b Dim. 

 The data may be labelled (TBD).

 */

class Data{

  /// Number of observation vectors
  uint NbPts; 
  /// Dimension
  uint Dim ; 
  /// Observations 
  vector<double> X;
  /// Labels
  vector<uint> labels;
  
public :

  /// Default constructor
  Data(); 
  /// Constructor
  Data(uint nbp, uint nbv); 
  /// Destructor
  ~Data();
  
  // Getters

  /// Get the number of observations
  uint Get_N();
  /// Get the dimension 
  uint Get_D(); 

  /// Get the observation at a given site i
  void Get(uint i, vector<double> & obs );
  /// Get X_i[d]
  double Get(uint i, uint d);

  /// Get the label of site i 
  uint GetLabel(uint i);
  /// Get all the labels
  void GetLabels(vector<uint> & labs);
  // Setters

  /// Set the observation at the site i
  void Set(uint i, vector<double> const &   obs );
  /// Set the label of site i 
  void SetLabel(uint i, uint lab );

  // IO

  /// read data from a file, dimension = 1 
  void ReadFromFile(string filename); 
  /// read data in file , dimension = dim
  void ReadFromFile(string filename, uint dim);  
  /// Write data in file
  void WriteToFile(string filename); 

  /// Compute the (empirical) mean and covariance of the data 
  void Stats(vector<double> & mean, vector<double> & cov);

  /// K-means algorithm
  void K_means(uint K, vector<double> & centers, vector<uint> & labels);

  /// Display some (statistical) information about the dataset
  void Info();
  
};

#endif
