#define __cplusplus__
#ifndef __DIAGNORMAL__
#define __DIAGNORMAL__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>


#include <Spatial_Data.hpp>
#include <Distribution.hpp>

using namespace std;


/*! \class Diag_Normal
 *  \brief A class representing a multidimensional normal distribution with a diagonal covariance matrix.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

*/

class Diag_Normal: public Distribution {
  /// means
  vector<double> Mu; 
  /// Covariance matrices 
  vector<double> Sigma;

public :
  Diag_Normal();
  /// Constructor of a centered reduced ( diagonal ) normal in dimension dim
  Diag_Normal(uint dim);
  /// Constructor of a diagonal  normal in dimension dim with given mean and variance
  Diag_Normal(uint dim, vector<double> const &  mu, vector<double> const &  sigma);

  ~Diag_Normal();

  /// Free all used memory. 
  void DeleteAll();
  
  /// Simulate
  void Simulate(vector<double> & X);

	/// Compute the log density 
  double LogDensity(vector<double> const &  X);


  /// Set the mean to the given vector
  void Set_Mu(vector<double> const &  m);
  /// Set the variance to the given vector of diagonal elements
  void Set_Sigma(vector<double> const &  sig);
  /// Get the mean
  void Get_Mu(vector<double> &mu);
  /// Get the variance
  void Get_Sigma(vector<double> &sigma);

  /// Estimate the mean from the given data and frequencies
  void Estimate_Mu(Spatial_Data *spatd, vector<double>  const &  freqs);
  /// Estimate the variance from the given data and frequencies
  void Estimate_Sigma(Spatial_Data *spatd, vector<double> const &   freqs);

  /// Estimate all parameters from the given data and frequencies
  void Estimate(Spatial_Data *spatd, vector<double>  const &  freqs);

  /// Estimate all parameters from the given data
  void Estimate(Spatial_Data *spatd);
  
  /// Read all parameters from a stream
  void Read_Params(istream & is);
  /// Write all parameters to a stream
  void Write_Params(ostream & os);
	/// info
	void Info();

  /// Number of free parameters
  uint Degrees_Of_Freedom();
};


#endif
