#define __cplusplus__
#ifndef __EFPOTTSMRF__
#define __EFPOTTSMRF__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

#include <MRF.hpp>

using namespace std;

/*! \class External_Potts
 *  \brief A class representing a Potts MRF Model with an external field.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

 A potts Hidden Markov Random Field model with external field V , with one parameter beta.

*/

class External_Potts : public MRF {
  /// Pair potential
  double Beta ; 
  /// Singleton potentials
  vector<double> Alpha;

public :
  /// Default constructor
  External_Potts();
  /// Constructor
  External_Potts(Neighborhood_System *nhs, uint k );
  /// Constructor
  External_Potts(Neighborhood_System *nhs, uint k , 
		       vector<double> const & alph,double bet);
  /// Default destructor
  ~External_Potts();
  
  /// Free all allocated memory.
  void DeleteAll();

  /// Get the singleton potentail
  void Get_Alpha(vector<double> & alpha);
  /// Get the pair potential
  double Get_Beta();
  /// Get the pair potential
  void Get_Beta(vector<double> & beta);  
    
  /// Hamiltonian
  double H(uint i, uint k, vector<uint>  const &  Z);
  /// Mean-field Hamiltonian
  double Hmf(uint i, uint k);
  
  /// Read the object definition from a file
  void ReadFromFile(string filename);
  /// Write the object definition from a file
  void WriteToFile(string filename);


  /// Compute the gradient.
  void Gradient(vector<double> const &  tik, vector<double> & Q);
  /// Update model parameters using the current value of the gradient
  void Update_Gradient(vector<double> const &   Q);

  /// Read all parameters from a stream
  void Read_Params(istream & is);
  ///  Write the model parameters to the stream os.
  void Write_Params(ostream & os);

  /// Number of free parameters
  uint Degrees_Of_Freedom();
  
  /// Display info on the model.
  void Info();
  
};

#endif
