#define __cplusplus__
#ifndef __GENPOTTSMRF__
#define __GENPOTTSMRF__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

#include <MRF.hpp>

using namespace std;



/*! \class Generic_Potts
 *  \brief A class representing a generic Potts MRF Model.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

 A potts Hidden Markov Random Field model with external field Alpha , with an interaction matrix parameter beta. This model is used by JB.

*/

class Generic_Potts : public MRF {
  /// Pair potential
  vector<double> Beta ; 
  /// Singleton potentials
  vector<double> Alpha;

public :
  /// Default constructor
  Generic_Potts();
  /// Constructor
  Generic_Potts(uint k);
  /// Constructor
  Generic_Potts(Neighborhood_System *nhs);
  /// Constructor
  Generic_Potts(Neighborhood_System *nhs, uint k);
  /// Constructor
  Generic_Potts(Neighborhood_System *nhs, uint k , 
		       vector<double> const &  alph ,vector<double>  const &  bet);
  /// Default destructor
  ~Generic_Potts();
  /// Free all memory used by the object.
  void DeleteAll();

  /// Get the singleton potentail
  void Get_Alpha(vector<double> & a);
  /// Get the pair potentials
  void Get_Beta(vector<double> & b );
  
	/// Hamiltonian
  double H(uint i, uint k, vector<uint>  const &  Z);
  /// Mean-field Hamiltonian
  double Hmf(uint i, uint k);
  
  /// Read the object definition from a file
  void ReadFromFile(string filename);
  /// Write the object definition from a file
  void WriteToFile(string filename);

  /// Compute the gradient given the fuzzy classifictaion.
  void Gradient(vector<double> const &  tik, vector<double> & Q);
  /// Update the gradient
  void Update_Gradient(vector<double> const &   Q);  

  void Read_Params(istream & is);
  void Write_Params(ostream & os);

  /// Number of free parameters
  uint Degrees_Of_Freedom();

  void Info();

};

#endif
