#define __cplusplus__
#ifndef __HMRF__
#define __HMRF__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>


#include <Seg_Model.hpp>
#include <Spatial_Data.hpp>
#include <Neighborhood_System.hpp>
#include <MRF.hpp>
#include <Distribution.hpp>
#include <Diag_Normal.hpp>
#include <Normal.hpp>
#include <HighDim_Normal.hpp>
#include <Laplace.hpp>


using namespace std;


/*! \class HMRF
 *  \brief A class representing a hidden Markov random field.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

 Abstract class.

*/

class HMRF  : public Seg_Model{

protected :

  /// (hidden) MRF
  Random_Field *RF;
  
public :

	/// Constructor
	HMRF();
	/// Constructor
	HMRF(Neighborhood_System *nhs);
	/// Constructor
	HMRF(uint k, uint dim , Neighborhood_System *nhs);
	/// Constructor
	HMRF( Random_Field *rf , vector<Distribution *> const &  distrib);
  /// Constructor
	HMRF( Random_Field *rf , vector<Diag_Normal *> const &  distrib);
	/// Constructor
	HMRF( Random_Field *rf , vector<Normal *> const &  distrib);
	/// Constructor
	HMRF( Random_Field *rf , vector<Laplace *> const &  distrib);
	/// Constructor
	HMRF( Random_Field *rf , vector<HighDim_Normal *> const &  distrib);

	/// Destructor.
  ~HMRF();

  /// Get the number of sites
  uint Get_N();

  /// Likelihood
  double LogLikelihood(Spatial_Data *spatd);
  /// BIC -- Bayesian Information Criterion
  double BIC(Spatial_Data *spatd );
  /// ICL --  Integrated Completed Likelihood
  double ICL(Spatial_Data *spatd );

  /// Hamiltonian
  double H(uint i, uint k, vector<uint> const &  z);
  /// Mean-field hamiltonian,  \f$ H \f$
  double Hmf(uint i, uint k);

  void Compute_PGmf(vector<double> & P_G);
  /// Set the mean-field neighborhood, \f$ \tilde Z \f$
  void Set_Zmf( vector<double> const &  z );
  /// Set the mean-field neighborhood,  \f$ \tilde Z  \f$ at site i
  void Set_Zmf( vector<double> const &  z , uint i);
  /// Get the mean-field neighborhood, \f$ \tilde Z \f$
  void Get_Zmf( vector<double> & z );

	/// Simulation procedure	
	void Simulate(Data *dat, uint gibbs_sampler_nbiter );
  /// Simulation procedure
	void Simulate(Data *dat, double grad_tol );

  ///Set gradient descent parameters 
  void Set_Gradient(double step, double tol, uint maxiter);

  /// Estimate the MRF parameters.
  void Estimate_MRF(vector<double>  const &  tik); 
  /// Step M of the NR-EM algorithm
  void Estimate(Spatial_Data *spatd, vector<double>  const &  tik);
  
  /// Completed likelihood
  double Completed_LogLikelihood(Spatial_Data *spatd, vector<double> const & tik);

  /// PLIC -- Pseudolikelihood Information Criterion
  double PLIC(Spatial_Data *spatd, vector<uint> const & Z_ICM); 

	/// Degrees of freedom
	uint Degrees_Of_Freedom() ;

	/// Information
	void Info();
	/// Read from file
	void ReadFromFile(string filename);
	/// Write to file
	void WriteToFile(string filename);

};


#endif
