#define __cplusplus__
#ifndef __HMRFALGO__
#define __HMRFALGO__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>


#include <Seg_Algorithm.hpp>
#include <HMRF.hpp>


using namespace std;


/*! \class HMRF_Algorithm
 *  \brief A class representing an abstract HMRF algorithm.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005


*/

class HMRF_Algorithm : public Seg_Algorithm{

protected :
	/// Data used to estimate the model.
  Spatial_Data *SpatData;
  
  /// Model on which we will apply the algorithm
  HMRF *Model;
  
	/// Update log-density
	void Update_Density();

	/// Log density
	vector<double> logdens;
  
public :
	/// Destructor
	virtual ~HMRF_Algorithm();

  /// Get the number of points
	uint Get_N();

	/// Get the number of classes
	uint Get_K();
	
	/// Run for a number of iterations ...
  virtual void Run(uint nbiter);
  /// E-step
  void E_Step();
  /// M-step
  void M_Step();

  /// Display some info on the algorithm
  void Info();

  /// Compute current MAP classification
  void Compute_MAP_Labels(vector<uint> & Labels);
  
  virtual void Compute_fuzzy(uint i,vector<double> &probs);

  /// Initialize the algorithm given the labels
  void Init_Labels(vector<uint> const & labs);
  /// Initialize with kmeans
  void Init_KMeans();
  /// Initialize the algorithm with random labels
  void Init_Random();
  /// Initialize the algorithm given the fuzzy classification
  void Init_fuzzy(vector<double> const & t);

  /// Write the fuzzy classification in a file
  void Write_fuzzy(string filename);

  // Stopping criteria
  /// Completed likelihood criterion
  void Compute_CLCrit();
  /// Fuzzy classification criterion 
  void Compute_FuzzyCrit();
  /// Hard classification criterion
  void Compute_HardCrit();

  
};


#endif
