#define __cplusplus__
#ifndef __HDNORMAL__
#define __HDNORMAL__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>


#include <Spatial_Data.hpp>
#include <Distribution.hpp>

using namespace std;


/*! \class Normal
 *  \brief A class representing a multidimensional normal distribution.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

*/

class HighDim_Normal: public Distribution{
  
  /// means (dimension Dim)
  vector<double> Mu; 
  /// reduced dimension
  uint Dim_red;
  /// high eigenvalue
  double A;
  /// small eigenvalue
  double B;
  /// eigenvectors related to a (dimension Dim*Dim_red)
  vector<double> Q;


public :
  /// Constructor of a centered reduced normal in dimension 1
  HighDim_Normal();
  /// Constructor of a centered reduced normal in dimension dim
  HighDim_Normal(uint dim);
  /// Constructor of a normal, mean=(0...0), Q=(e1,...,e1) in dimension dim, reduced dimension dim_red, eigenvalues a and b
  HighDim_Normal(uint dim, uint dim_red, double a, double b);
  /// Constructor of a normal, mean=mu, Q=q in dimension dim, reduced dimension dim_red, eigenvalues a and b
  HighDim_Normal(uint dim, uint dim_red, double a, double b, vector<double> const &  mu, vector<double> const &  q);

  /// Destructor
  ~HighDim_Normal();

  /// Free all used memory. 
  void DeleteAll();
  
  /// info
  void Info();

  /// Get the reduced dimension
  uint Get_Dred();

  /// Get the mean
  void Get_Mu(vector<double> &mu);

  /// Get A (high eigenvalue)
  double Get_A();

  /// Get B (small eigenvalue)
  double Get_B();

  /// Get Q (dimension Dim*Dim_red)
  void Get_Q(vector<double> &q);

  /// Get Q completed with zero (dimension Dim*Dim)
  void Get_Qc(vector<double> &qc);

  /// Set the mean
  void Set_Mu(vector<double>  const & m);

  /// Set A
  void Set_A(double a);
 
  /// Set B
  void Set_B(double b);
    
  /// Set Q
  void Set_Q(vector<double> const &  q);  

  /// Compute Delta=diag(A,...,A,B,...,B)
  void Compute_Delta(vector<double> Delta);

  ///Compute Sigma=Q Delta tQ
  void Compute_Sigma(vector<double> Sigma);

  /// Simulate
  void Simulate(vector<double> & X);

  /// Get projection of X onto the affine space generated by A and passing throw Mu 
  void Proj(vector<double> const &X, vector<double> &pX);

 /// Compute the log density 
  double LogDensity(vector<double> const &  X);
  
  /// Compute the empirical covariance from the given data and frequencies
  void Compute_Empirical_Sigma(Spatial_Data *spatd, vector<double>  const &  freqs, vector<double> &Sigma);

  /// Estimate the mean from the given data and frequencies
  void Estimate_Mu(Spatial_Data *spatd, vector<double>  const &  freqs);

  /// Estimate the reduced dimension, eigenvalues A and B, eigenvectors Q
  void Estimate_dabQ(Spatial_Data *spatd, vector<double>  const &  freqs);

  /// Estimate all parameters from the given data
  void Estimate(Spatial_Data *spatd, vector<double>  const &  freqs);

  /// Number of free parameters
  uint Degrees_Of_Freedom();

  /// Read all parameters from a stream
  void Read_Params(istream & is);

  /// Write all parameters to a stream
  void Write_Params(ostream & os);

};


#endif
