#define __cplusplus__
#ifndef __IIDMIX__
#define __IIDMIX__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>



#include <Spatial_Data.hpp>
#include <Seg_Model.hpp>
#include <Diag_Normal.hpp>
#include <Normal.hpp>
#include <HighDim_Normal.hpp>
#include <Laplace.hpp>


using namespace std;


/*! \class IID_Mixture
 *  \brief A class representing an IID mixture.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

 Abstract class.

*/

class IID_Mixture : public Seg_Model{

protected:
  /// mixture probabilities
  vector<double> Pi;

public :

	/// Constructor
	IID_Mixture();
	/// Constructor
	IID_Mixture(uint k, uint dim);
	/// Constructor
	IID_Mixture(uint k, vector<Distribution *> const &  distrib);
  /// Constructor
	IID_Mixture(uint k, vector<Diag_Normal *> const &  distrib);
	/// Constructor
	IID_Mixture(uint k, vector<Normal *> const &  distrib);
	  /// Constructor
	IID_Mixture(uint k, vector<HighDim_Normal *> const &  distrib);
	/// Constructor
	IID_Mixture(uint k, vector<Laplace *> const &  distrib);
	/// Destructor
  ~IID_Mixture();

  /// Get the mixture probabilities.
  void Get_Pi( vector<double> & pi );
  /// Set the mixture probabilities.
  void Set_Pi( vector<double>  const &  pi );
  
  /// Likelihood
  double LogLikelihood(Spatial_Data *spatd);
  /// Completed Likelihood
  double Completed_LogLikelihood(Spatial_Data *spatd,  vector<double>  const &  tik );

  /// BIC -- Bayesian Information Criterion
  double BIC(Spatial_Data *spatd );
  /// ICL --  Integrated Completed Likelihood
  double ICL(Spatial_Data *spatd );

  /// Estimate mixture probabilities
  void Estimate_Pi(vector<double>  const &  tik);
	/// Step M of the EM algorithm
  void Estimate(Spatial_Data *spatd, vector<double>  const &  tik);

	/// Simulation procedure
	void Simulate(uint N, Data *dat);
	/// Degrees of freedom
	uint Degrees_Of_Freedom();

	/// Read parameters from file
	void ReadFromFile(string filename);
	/// Write parameters to file
	void WriteToFile(string filename);
	/// Display info
	void Info();

  
};


#endif
