#define __cplusplus__
#ifndef __LAPLACE__
#define __LAPLACE__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

#include <Spatial_Data.hpp>
#include <Distribution.hpp>

using namespace std;


/*! \class Laplace
 *  \brief A class representing a multidimensional Laplace distribution.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

*/


class Laplace: public Distribution{
  
  /// Position parameter
  vector<double> Mu; 
  /// Shape parameter 
  vector<double> Lambda;

public :
  /// Construtor
  Laplace();
  /// Construtor
  Laplace(uint dim);
  /// Construtor
  Laplace(uint dim, vector<double> const &  mu, vector<double> const &  lambda);

  /// Destrutor
  ~Laplace();

  /// Free all used memory. 
  void DeleteAll();

  /// Simulate
  void Simulate(vector<double> & X);

  /// Compute the log density 
  double LogDensity(vector<double> const &  X);

  /// Set the mean/position parameter
  void Set_Mu(vector<double> const &  m);
  /// Set the shape paramter
  void Set_Lambda(vector<double> const &  lam);

  /// Estimate the mean/position parameter
  void Estimate_Mu(Spatial_Data *spatd, vector<double> const &   freqs);
  /// Estimate the shape paramter
  void Estimate_Lambda(Spatial_Data *spatd, vector<double>  const &  freqs);

  /// Stimate all the parameters
  void Estimate(Spatial_Data *spatd, vector<double> const &   freqs);

  /// Initialize at random
  void Init_at_random(Spatial_Data *spatd);

  /// Read all parameters from a stream
  void Read_Params(istream & is);
  /// Write all parameters to a stream
  void Write_Params(ostream & os);

  /// Number of free parameters
  uint Degrees_Of_Freedom();

	/// info
	void Info();

};



#endif
