/*! \file LinAlg.hpp
    \brief An include file containing the definition of linear algebra functions used here.
    
*/

#define __cplusplus__
#ifndef __LINALG__
#define __LINALG__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

using namespace std;

// Trace of a square matrix
double Trace(vector<double> const &  M);

/// Cholesky decomposition , M = L * L^T
void Cholesky(vector<double> const &  M, vector<double> & L);

/// Determinant
double Determinant(vector<double> const &  L);
/// Matrix invert , M1 = M ^(-1) 
void Matrix_invert(vector<double> const &  M, vector<double> & M1);

/// Square matrix multiplication C = A * B  
void Matrix_mult(vector<double> const &  A, vector<double> const &   B, vector<double> & C);

/// matrix vector multiplication. Y = A * X
void Matrix_Vec_mult( vector<double> const &  A, vector<double> const &   X, vector<double> & Y);

/// vector aaddition. v3 = v1 + v2
void Vec_add( vector<double> const &  v1, vector<double> const & v2, vector<double> & v3);

/// vector subtraction. v3 = v1 - v2
void Vec_sub( vector<double> const &  v1, vector<double> const & v2, vector<double> & v3);
 
///vector copy. v2 = v1
void Vec_copy(vector<double> const &  v1, vector<double> & v2);

/// Square matrix transpose B = A^T  
void Matrix_transpose(vector<double> const &  A, vector<double> &  B );

/// Print a matrix
void Matrix_print(vector<double> const &  A );


/// Matrix L2 distance 
double Matrix_dist(vector<double> const &  A, vector<double> const &   B );
/// Norm in L2
double norm(vector<double> const &  A);
/// Norm² in L2
double norm2(vector<double> const &  A);
/// Norm² in L2
uint norm2(vector<uint> const &  A);
/// Create a random square matrix
void Matrix_random(uint N, vector<double> & A );

/// Create a zero square matrix
void Matrix_setzero(uint N, vector<double> & A );

/// Create a identity  matrix
void Matrix_setid(uint N, vector<double> & A );


#endif
