#define __cplusplus__
#ifndef __MRF__
#define __MRF__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

#include <Neighborhood_System.hpp>
#include <Random_Field.hpp>

using namespace std;

/*! \class MRF
 *  \brief A class representing an MRF.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

 Generic Markov Random Field model. Abstract. 


*/

class MRF: public Random_Field{

protected :
  /// Number of classes.
  uint K;

	/// Precompute mean-field sums of \tilde Z (for the gradient algo.)
  void Precompute_Smf();

public :

  /// Destructor
  virtual ~MRF();


  /// Get the number of classess/colors ...
  uint Get_K();

  /// Hamiltonian
  virtual double H(uint i, uint k, vector<uint>  const &  Z);
  /// Mean-field hamiltonian, \f$ \tilde H \f$ 
  virtual double Hmf(uint i, uint k);

  /// compute the pseudo-likelihood for data  \f$ Z \f$
  double PL(vector<uint> const &   Z);

  /// Compute the gradient
  virtual void Gradient(vector<double> const & tik, vector<double> & Q);  
  /// Update model parameters in the gradient descent
  virtual void Update_Gradient(vector<double> const &   Q);

  /// Read the model parameters from the stream is.
  virtual void Read_Params(istream & is);
  /// Write the model parameters to the stream os.
  virtual void Write_Params(ostream & os);

  /// Number of free parameters
  virtual uint Degrees_Of_Freedom();

  /// Print some info on the model.
  virtual void Info();
};

#endif
