#define __cplusplus__
#ifndef __MIXALGO__
#define __MIXALGO__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

#include <Seg_Algorithm.hpp>
#include <IID_Mixture.hpp>

using namespace std;


/*! \class Mixture_Algorithm
 *  \brief A class representing an abstract IID mixture algorithm.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005


*/

class Mixture_Algorithm : public Seg_Algorithm{

protected :
	/// Data used to estimate the model.
  Spatial_Data *SpatData;

  /// Model on which we will apply the algorithm
  IID_Mixture *Model;

	/// Update log-density
	void Update_Density();

	/// Log density
	vector<double> logdens;

public :
	/// Destructor
	virtual ~Mixture_Algorithm();
	
  /// Get the number of points
	uint Get_N();

	/// Get the number of classes
	uint Get_K();
	
  /// Display info
  void Info();
  /// Run algo
  void Run(uint nbiter);
  /// E-step
  void E_Step();
  /// M_Step
  void M_Step();
  /// Compute MAP labels
  void Compute_MAP_Labels(vector<uint> & Labels);
  /// Initialize with labels
  void Init_Labels(vector<uint> labs);
  /// Initialize with kmeans
  void Init_KMeans();

  /// Init the algorithm at random
  void Init_Random();
  
  /// Write the fuzzy classification in a file
  void Write_fuzzy(string filename);
  /// Compute fuzzy classification
  virtual void Compute_fuzzy(uint i,vector<double> &probs);

  // Stopping criteria
  /// Completed likelihood criterion
  void Compute_CLCrit();
  /// Fuzzy classification criterion 
  void Compute_FuzzyCrit();
  /// Hard classification criterion
  void Compute_HardCrit();

};


#endif
