#define __cplusplus__
#ifndef __NORMAL__
#define __NORMAL__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>


#include <Spatial_Data.hpp>
#include <Distribution.hpp>

using namespace std;


/*! \class Normal
 *  \brief A class representing a multidimensional normal distribution.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

*/

class Normal:public Distribution{
  
  /// means
  vector<double> Mu; 
  /// Covariance matrices 
  vector<double> Sigma;


  /// inverse square root of covariance matrix 
  vector<double> Sigma_inv_sqrt;
  /// determinant of square root of covariance matrix 
  double Sigma_sqrt_det;



public :
  /// Constructor of a centered reduced normal in dimension 1
  Normal();
  /// Constructor of a centered reduced normal in dimension dim
  Normal(uint dim);
  /// Constructor of a normal with given  mean and variance
  Normal(uint dim, vector<double> const &  mu, vector<double> const &  sigma);

  /// Destructor
  ~Normal();

  /// Free all used memory. 
  void DeleteAll();
  
  /// Get the mean
	void Get_Mu(vector<double> &mu);	
  /// Get the variance matrix
	void Get_Sigma(vector<double> &sigma);
	
  /// Precompute cholesky decomposition for all class covariance matrices
  void precompute_cholesky();

  /// Simulate
  void Simulate(vector<double> & X);

 	/// Compute the log density 
  double LogDensity(vector<double> const &  X);

  /// Set the mean to the given vector
  void Set_Mu(vector<double> const &  m);
  /// Set the variance to the given vector of diagonal elements
  void Set_Sigma(vector<double>  const & sig);

  /// Estimate the mean from the given data and frequencies
  void Estimate_Mu(Spatial_Data *spatd, vector<double>  const &  freqs);
  /// Estimate the variance from the given data and frequencies
  void Estimate_Sigma(Spatial_Data *spatd, vector<double>  const &  freqs);

  /// Estimate all parameters from the given data and frequencies
  void Estimate(Spatial_Data *spatd, vector<double>  const &  freqs);

  /// Estimate all parameters from the given data
  void Estimate(Spatial_Data *spatd);
  
  /// Read all parameters from a stream
  void Read_Params(istream & is);
  /// Write all parameters to a stream
  void Write_Params(ostream & os);

  /// Number of free parameters
  uint Degrees_Of_Freedom();

	/// info
	void Info();
};


#endif
