#define __cplusplus__
#ifndef __SEGALGO__
#define __SEGALGO__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

#include <Seg_Model.hpp>

using namespace std;

#define ITER_CRIT (0)
#define CL_CRIT (1)
#define FUZZY_CRIT (2)
#define HARD_CRIT (3)
#define MAX_CRIT (4)


/*! \class Seg_Algorithm
 *  \brief A class representing a segmentation algorithm.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

 Abstract class.

*/

class Seg_Algorithm{

protected :
  
  /// Fuzzy classification
  vector<double> fuzzy;

  /// All values of stopping criteria
  vector<double> Crit_Values;
  /// Current (chosen) stopping criterion
  uint Criterion;
  /// limit value of the stopping criterion
  double Tol;
  
  /// Old value of completed likelihood
  double Old_CL;
  /// Old value of fuzzy classification
  vector<double> old_fuzzy;
  /// Old map labels
  vector<uint> old_map_labs;
  
public :
  /// Get the number of points
	virtual uint Get_N();

	virtual uint Get_K();
  
  virtual ~Seg_Algorithm();
  
	/// Initialize with labels
  virtual void Init_Labels(vector<uint> const &  labs);
  /// Initialize randomly
  virtual void Init_Random();

  /// True if the algorithm has converged (the criterion has reached the limit)
  bool HasConverged();

  /// Compute MAP labels -- segmentation result.
  virtual void Compute_MAP_Labels(vector<uint> & Labels);
  
  virtual void Compute_fuzzy(uint i,vector<double> &probs);
  
  // Stopping criteria
  /// Completed likelihood criterion
  virtual void Compute_CLCrit();
  /// Fuzzy classification criterion 
  virtual void Compute_FuzzyCrit();
  /// Hard classification criterion
  virtual void Compute_HardCrit();

  /// display info about the criteria
  void Display_Stopping_Rule();
  void Get_Stopping_Crit(vector<double> &crit);
  
  /// Recompute criteria values
  void Update_Stopping_Rules();
  /// Set the criterion to the  default
  void Use_Default_Stopping_Rule();
  /// Set the criterion to a given one
  void Set_Stopping_Rule(uint crit, double tol);

  /// Display some runtime information ...
  virtual void Info();
  
  /// Run for a number of iterations ...
  virtual void Run(uint nbiter);
  /// Run until the convergence test is ok ...
  void RunUntilConvergence();
  /// M-step
  virtual void M_Step();
  /// E-step 
  virtual void E_Step(); 
      
};


#endif
