#define __cplusplus__
#ifndef __SEGMOD__
#define __SEGMOD__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>


#include <Spatial_Data.hpp>
#include <Distribution.hpp>


/*! \class Seg_Model
 *  \brief A class representing a segmentation model.
 *  \author Lemine Abdallahi
 *  \date    Aug. 2005

 Abstract class.

*/


using namespace std;

class Seg_Model{

protected:

	/// Distributions
	vector <Distribution*> Distrib;

    
public :
	/// destructor
  virtual ~Seg_Model();

	/// Get dimension
	uint Get_D();
	/// Get the number of classes
	uint Get_K();

  /// Likelihood
  virtual double LogLikelihood(Spatial_Data *spatd);
  /// Completed likelihood
  virtual double Completed_LogLikelihood(Spatial_Data *spatd, vector<double>  tik);
  /// BIC -- Bayesian Information Criterion
  virtual double BIC(Spatial_Data *spatd );
  /// ICL --  Integrated Completed Likelihood
  virtual double ICL(Spatial_Data *spatd );
  
	/// Compute the log-density for observation X in class k
  double LogDensity(vector<double> const &  X, uint k);
  /// Compute the density for observation X in class k
	double Density(vector<double> const &  X, uint k);

	/// Estimate the distributions parameters.
	void Estimate_Distributions(Spatial_Data *spatd, vector<double>  const &  tik);

  /// Number of free parameters
  virtual uint Degrees_Of_Freedom();
  /// Display model information
  virtual void Info();
  /// Estimate model parameters
  virtual void Estimate(Spatial_Data *spatd, vector<double>  const &  tik);
};


#endif
