/*! \file Stat.hpp
    \brief An include file containing the definition of some generic statistical functions.
    
*/


#define __cplusplus__
#ifndef __STATS__
#define __STATS__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>

using namespace std;


///   generate a random number uniformly in [0,1].
double runif();


///   generate a random number following a standard gaussian distribution.
double rnorm();

///   generate a random vector of n IID gaussian variables.
void rnorm(uint n, vector<double> & result);

///   generate a random vector of a d-dimensional N( mu_d , sigma_d).
void rnorm(vector<double> const &  mu, vector<double> const &  sigma, vector<double> & result);


void rnorm_diag(vector<double> const &  mu, vector<double> const &  sigma,  vector<double> & result);

///    Sampling from a distribution (p1, ...., pn)
uint Sample(vector<double> const &  probs);

/// Weighetd median
double weighted_median(vector<double> const &  X , vector<double> const &  W);

/// argmax
uint argmax(vector<double> const &  X);



/// Maximum
void Maximum(vector<double> const &  X, uint D, vector<double> & m);
/// Minimum
void Minimum(vector<double> const &  X, uint D, vector<double> & m);
/// Mean and  Standard deviation
void Mean_SD(vector<double> const &  X, uint D, vector<double> & m, vector<double> & s);

/// statistical summary
void Summary(vector<double> const &  X, uint D);

/// choose k elements out of {0,1,....,N-1}
void choose(uint N, uint K, vector<uint> & indexes);


/// Exponential function
double Exp(double x);

/// Logarithm function
double Log(double x);



#endif
