/*! \file Utils.hpp
    \brief An include file containing the definition of some useful functions not directly linked with the main problem.

    For the moment, it contains some string conversion functions.
    
*/



#define __cplusplus__
#ifndef __UTILS__
#define __UTILS__

#include<iostream>
#include<istream>
#include<fstream>
#include<string>
#include<vector>
#include<sstream>
#include <cmath>


using namespace std;

/*!

\fn  T fromString(const std::string& s);

\brief read a variable of type T from a string representation s.

This is a shortcut to read any type T from a string s. It is a C++ equivalent for sscanf. An example use is :

\code
string s="123";
uint i=fromString<uint>(s); -> i=123
double x=fromSting<double>("3.145"); -> x=3.145
\endcode

*/

template<typename T> T fromString(const std::string& s) {
  std::istringstream is(s);
  T t;
  is >> t;
  return t;
}

/*!

\fn string  toString(const T & t);

\brief store the string representation of variable t in string s.

This is a shortcut to print any type T as a string s. It is a C++ equivalent for sprintf. An example use is :

\code
uint i=123;
string si=toString<uint>(i); -> si="123"
string sd=toString<double>(3.145); -> sd="3.145"
\endcode

*/



template<typename T> std::string toString(const T& t) {
  std::ostringstream s;
  s << t;
  return s.str();
}


/**
   Read a vector.
*/

template<typename T> void ReadVector(istream & is , std::vector<T>& tvec) {
  uint sz=0;
  is >> sz;
  tvec.clear();
  tvec.resize(sz);
  for( uint i=0 ; i < sz ; i++ ) {
    T t ;
    is >> t ;
    tvec[i]=t;
  }
}


/**
   Write a vector.
*/

template<typename T> void WriteVector(ostream & os , const std::vector<T> & tvec) {
  uint sz= tvec.size();
  os << sz;
  for( uint i=0 ; i < sz ; i++ ) {
    os << " " << tvec[i] ;
  }
  os << endl;
}


char upper(char c);


#endif
