#include <Stat.hpp>
#include <LinAlg.hpp>
#include <Utils.hpp>
#include <Generic_Potts.hpp>


// default constructor
Generic_Potts::Generic_Potts(){
  K=1;
  NHS=new Image_Local_Neighborhood();
  Alpha.resize(K);
  for(uint i=0;i<K;i++) Alpha[i]=0.0;
  Beta.resize(K*K);
  for(uint i=0;i<(K*K);i++) Beta[i]=0.0;
  for(uint i=0;i<K;i++) Beta[i+K*i]=0.1;

  uint N=NHS->Get_N();
  uint lmax=NHS->LMAX();

  double step_init=1.8/( N * (K+K*K*lmax) );
  uint nb_param=K+K*K;
 	gradient_step.clear();
	gradient_step.resize(nb_param,step_init);  
  
  gradient_maxiter=2000;
  gradient_tol=0.02;
  precomputed_smf=false;
	fix=false;
}

// default constructor
Generic_Potts::Generic_Potts(uint k){
  K=k;
  NHS=new Image_Local_Neighborhood();
  Alpha.resize(K);
  for(uint i=0;i<K;i++) Alpha[i]=0.0;
  Beta.resize(K*K);
  for(uint i=0;i<(K*K);i++) Beta[i]=0.0;
  for(uint i=0;i<K;i++) Beta[i+K*i]=0.1;
  uint N=NHS->Get_N();
  uint lmax=NHS->LMAX();  

  double step_init=1.8/( N * (K+K*K*lmax) );
  uint nb_param=K+K*K;
 	gradient_step.clear();
	gradient_step.resize(nb_param,step_init);  

  gradient_maxiter=2000;
  gradient_tol=0.02;
  precomputed_smf=false;
	fix=false;
}


// Constructor
Generic_Potts::Generic_Potts(Neighborhood_System *nhs){

  NHS=nhs;
  K=1;
  Alpha.resize(K);
  for(uint i=0;i<K;i++) Alpha[i]=0.0;
  Beta.resize(K*K);
  for(uint i=0;i<(K*K);i++) Beta[i]=0.0;
  for(uint i=0;i<K;i++) Beta[i+K*i]=0.1;
  uint N=NHS->Get_N();
  uint lmax=NHS->LMAX();   

  double step_init=1.8/( N * (K+K*K*lmax) );
  uint nb_param=K+K*K;
 	gradient_step.clear();
	gradient_step.resize(nb_param,step_init);   

  gradient_maxiter=2000;
  gradient_tol=0.02;
  precomputed_smf=false;
	fix=false;
}

// Constructor
Generic_Potts::Generic_Potts(Neighborhood_System *nhs, uint k){

  NHS=nhs;
  K=k;
  Alpha.resize(K);
  for(uint i=0;i<K;i++) Alpha[i]=0.0;
  Beta.resize(K*K);
  for(uint i=0;i<(K*K);i++) Beta[i]=0.0;
  for(uint i=0;i<K;i++) Beta[i+K*i]=0.1;

  uint N=NHS->Get_N();
  uint lmax=NHS->LMAX();  

  double step_init=1.8/( N * (K+K*K*lmax) );
  uint nb_param=K+K*K;
 	gradient_step.clear();
	gradient_step.resize(nb_param,step_init);  

  gradient_maxiter=2000;
  gradient_tol=0.02;
  precomputed_smf=false;
	fix=false;
}


// Constructor
Generic_Potts::Generic_Potts(Neighborhood_System *nhs, uint k, vector<double > const &  alph, vector<double> const &  bet){
  K=k;
  NHS=nhs;
  // singleton potentials set to zero if alph does not have the right size.
  if(alph.size()==K){
    Alpha.resize(K);
    for(uint i=0;i<K;i++) Alpha[i]=alph[i];
  }
  else{
    cerr << "Generic_Potts::Generic_Potts(Neighborhood_System *nhs, uint k, vector<double > alph, vector<double> bet) : " << endl << " Bad initialization. Setting external field to zero. " << endl;
    Alpha.resize(K);
    for(uint i=0;i<K;i++) Alpha[i]=0.0;
  }
  
  // The matrix of pair potentials is set to zero if bet does not have the right size.
  if(bet.size()==(K*K)){
    Beta.resize(K*K);
    for(uint i=0;i<(K*K);i++) Beta[i]=bet[i];
  }
  else{
    cerr << "Generic_Potts::Generic_Potts(Neighborhood_System *nhs, uint k, vector<double > alph, vector<double> bet) : " << endl << " Bad initialization. Setting pair potentials to zero. " << endl;
    Beta.resize(K*K);
    for(uint i=0;i<(K*K);i++) Beta[i]=0.0;
    for(uint i=0;i<K;i++) Beta[i+K*i]=0.1;
  }

  uint N=NHS->Get_N();
  uint lmax=NHS->LMAX(); 

  double step_init=1.8/( N * (K+K*K*lmax) );
  uint nb_param=K+K*K;
 	gradient_step.clear();
	gradient_step.resize(nb_param,step_init);  
  
  gradient_maxiter=2000;
  gradient_tol=0.02;
  precomputed_smf=false;
	fix=false;
}

// default destructor
Generic_Potts::~Generic_Potts(){
  DeleteAll();
}


void Generic_Potts::DeleteAll(){
  Beta.clear();
  Alpha.clear();
}

void Generic_Potts::Get_Alpha(vector<double> & a){
  a.clear();
  a.resize(K);
  for(uint i=0;i<K;i++) a[i]=Alpha[i];
}

void Generic_Potts::Get_Beta(vector<double> & b ){
  b.clear();
  b.resize(K*K);
  for(uint i=0;i<K*K;i++) b[i]=Beta[i];
}




void Generic_Potts::ReadFromFile(string filename){
  ifstream ifs(filename.c_str());
  if(!ifs.good()){
    cerr << "Generic_Potts::ReadFromFile : " << endl;
    cerr << "Problem with file : " << filename << endl;
    cerr << "Exiting." << endl;
    exit(1);
  }

	bool read_error=false;	
	
  
  if(!(ifs >> K)) read_error=true;
  
  ReadVector<double>(ifs,Alpha);
  ReadVector<double>(ifs,Beta);
  
  if(read_error){
    cerr << "Generic_Potts::ReadFromFile : " << endl;
    cerr << "Problem with file : " << filename << endl;
    cerr << "Possible format error. Exiting." << endl;
    exit(1);    
  }
  

}



void Generic_Potts::WriteToFile(string filename){
  ofstream ofs(filename.c_str());
  if(!ofs.good()){
    cerr << "Generic_Potts::WriteToFile : " << endl;
    cerr << "Problem with file : " << filename << endl;
    cerr << "Exiting." << endl;
    exit(1);
  }

  bool write_error=false;
  
  if(!(ofs << K)) write_error=true;
  ofs << endl;
  WriteVector<double>(ofs,Alpha);
  WriteVector<double>(ofs,Beta);
  
  if(write_error){
    cerr << "Generic_Potts::WriteToFile : " << endl;
    cerr << "Problem with file : " << filename << endl;
    cerr << "Possible format error. Exiting." << endl;
    exit(1);    
  }

}

/**

Compute
\f$ H_ik(Z) = \alpha_k + \sum_{j~i} \sum_m \beta{km} 1_{Z_j=m} \f$
*/


double Generic_Potts::H(uint i , uint k,  vector<uint>  const &  Z){
  
  vector< Weighted_Neighbor * >  ineighs;
  ineighs.clear();
  NHS->GetNeighbors(i,ineighs);
  uint nv=ineighs.size();
  double pot_k=0.0;

  for (uint v = 0; v<nv ; v++){
    
    uint    j   = ineighs[v]->Index;
    //double w_ij = ineighs[v]->Weight;
    for (uint l = 0; l < K ; l++){
      double z_jl=0;
      if(Z[j]==l) z_jl=1;
      pot_k+= Beta[k*K+l]*z_jl;
    }    
  }
  pot_k = pot_k + Alpha[k];
  ineighs.clear();
  return(pot_k);
}

/**

Compute
\f$ H_ik(\tilde Z) = \alpha_k + \sum_{j~i} \sum_m \beta_{km} \tilde Z_{jm} \f$

with \f$ \tilde Z = Z^{mf} \f$

*/


double Generic_Potts::Hmf(uint i, uint k){
  double e=0.0;

  if( precomputed_smf ){
    double sum=0.0;
    for(uint m=0; m<K ; m++){
      sum += Beta[k*K+m] * S_mf[i*K+m]; 
      //if (sum>500) cerr << "Warning Generic_Potts::Hmf(uint i, uint k): sum=" << sum << " " << "alph=" << Alpha[k] << " " << "Beta=" << Beta[k*K+m] << " " << "S_mf=" << S_mf[i*K+m] << endl; 
    }
    e= Alpha[k] +  sum;
  }
  else{
    vector< Weighted_Neighbor * >  ineighs;
    NHS->GetNeighbors(i,ineighs);
    uint nv=ineighs.size();
    double sum=0.0;
    for(uint v=0;v<nv;v++)
      for(uint m=0; m<K ; m++){
				uint j=ineighs[v]->Index;
				sum += Beta[k*K+m] * Z_mf[j*K+m];
				//if (sum>500) cerr << "Warning Generic_Potts::Hmf(uint i, uint k): nv=" << nv << " " << "sum=" << sum << " " << "alph=" << Alpha[k] << " " << "Beta=" << Beta[k*K+m] << " " << "Z_mf=" << Z_mf[i*K+m] << endl; 
      }
    e= Alpha[k] +  sum;
    ineighs.clear();
  }

  return(e);
}





void Generic_Potts::Gradient(vector<double> const &  t, vector<double> & Q){

  //  cerr << "Generic_Potts::Gradient() ." << endl;

  if(!precomputed_smf) Precompute_Smf();
  uint N=NHS->Get_N();

  Q.clear();
  Q.resize(K+K*K);

  vector<double> P_G;
  Compute_PGmf(P_G);
  //  WriteVector<double>(cerr,P_G);

  for(uint k=0; k<K ; k++){
    double sum_i=0;
    for(uint i=0;i<N;i++) sum_i += t[i*K+k] - P_G[i*K+k];
    Q[k]=sum_i;
  }
  for(uint p=0; p<K ; p++){
    for(uint r=0; r<K ; r++){
      if(r!=p){
        double sum_i=0;
        for(uint i=0;i<N;i++){
          double sum_zir=S_mf[i*K+r];
          double sum_zip=S_mf[i*K+p];
          sum_i += sum_zir*(t[i*K+p]-P_G[i*K+p]) + sum_zip*(t[i*K+r]-P_G[i*K+r]);
        }
        Q[K+p*K+r]=sum_i;
      }
      else{
        double sum_i=0;
        for(uint i=0;i<N;i++){
          double sum_zip=S_mf[i*K+p];
          sum_i += sum_zip*(t[i*K+p] - P_G[i*K+p]);
        }
        Q[K+p*K+p]=sum_i;       
      }
    }
  }
  P_G.clear();
}


void Generic_Potts::Update_Gradient(vector<double> const & Q){
	//uint N=NHS->Get_N();
  //uint lmax=NHS->LMAX();
	//double step=1.8/( N * (K+K*K*lmax) );
	//for(uint k=0 ; k < K ; k++) Alpha[k]= Alpha[k] + step * Q[k];
	//for(uint kidx=0 ; kidx < (K*K) ; kidx++) Beta[kidx]= Beta[kidx] + step * Q[K+kidx];  
  for(uint k=0 ; k < K ; k++) Alpha[k]= Alpha[k] + gradient_step[k] * Q[k];
  for(uint kidx=0 ; kidx < (K*K) ; kidx++) Beta[kidx]= Beta[kidx] + gradient_step[K+kidx] * Q[K+kidx];  
  
  
}


/// Number of free parameters
uint Generic_Potts::Degrees_Of_Freedom(){
  return(K + (K*(K+1))/2);
}


void  Generic_Potts::Read_Params(istream & is){
  ReadVector<double>(is,Alpha);
  K=Alpha.size();
  ReadVector<double>(is,Beta);
  if(Beta.size()!=(K*K)){
    cerr << "Generic_Potts::Read_Params() " << endl;
    cerr << "Error reading  parametrs." << endl;
    exit(0);
  }
}

void  Generic_Potts::Write_Params(ostream & os){
  WriteVector<double>(os,Alpha);
  WriteVector<double>(os,Beta);
}

void Generic_Potts::Info(){
  cout << "Generic Potts :" << endl;
  cout << "K = " << K << endl;
  cout << "Alpha = " ;
  WriteVector<double>(cout,Alpha);
  cout << "Beta = " << endl;
  WriteVector<double>(cout,Beta);
  //  NHS->Info();
}
