#include <Stat.hpp>
#include <LinAlg.hpp>
#include <Utils.hpp>
#include <MRF.hpp>

/**

Destructor.

*/
MRF::~MRF(){

}


/**
Get the number of classes/colors.

@return The number of classes/colors in the MRF.

*/

uint MRF::Get_K(){
  return(K);
}

void MRF::Precompute_Smf(){
    //  cerr << "MRF::precompute_Smf() ." << endl;
  uint K=Get_K();
  uint N=NHS->Get_N();
  S_mf.clear();
  S_mf.resize(K*N);
  
  for(uint i=0 ; i < N ; i++){

    vector< Weighted_Neighbor * >  ineighs;
    NHS->GetNeighbors(i,ineighs);
    uint nv=ineighs.size();
    for(uint k=0 ; k < K ; k++){
      double sum=0.0;
      for(uint j=0;j<nv;j++){
        uint neidx=ineighs[j]->Index;
        sum += Z_mf[neidx*K+k];
      }
      S_mf[i*K+k]=sum;
      ineighs.clear();
    }
  }

}


/**

Compute the Hamiltonian 

@param  i : a site.
@param  k : a class.
@param  z : current state.

@return The value of the hamiltonian.

Abstract. Model-dependent.

*/


double MRF::H(uint i, uint k, vector<uint>  const &  Z){
  cerr << "Warning! MRF::H() is virtual." << endl;
  return(0.0);
}

/**

Compute the Hamiltonian mean-field approximation, given \f$ \tilde Z \f$

@param  i : a site.
@param  k : a class.

@return The value of the approximate hamiltonian.


Abstract. Model-dependent.

*/



double MRF::Hmf(uint i, uint k){
  cerr << "Warning! MRF::Hmf() is virtual." << endl;
  return(0.0);
}



/**

Compute the pseudo-Likelihhod of a given realization \b Z.

\f$ PL(Z)=\sum_i log P[Z_i=z_i | Z_{N_i}] \f$

 @param[in] Z : current state.

*/

double MRF::PL(vector<uint>  const &  Z){
  uint N=NHS->Get_N();
  double pl=0.0;
  for(uint i=0; i<N; i++){
    vector<double> probs;
    Compute_PG(i, Z, probs);
    double pli=Log(probs[Z[i]]);
    cerr << "check : " << i << " " << Z[i] << " " ;
    WriteVector<double>(cerr,probs);
    probs.clear();
    pl+=pli;
  }
  return(pl);
}



/**

Updates the model parameters in the gradient descent.

 @param[in] Q : current gradient value.

Abstract. Model-dependent.

*/


void MRF::Update_Gradient(vector<double>  const &  Q){
  
  cerr << "Warning! MRF::Update_Gradient(vector<double>  const &  Q) is virtual." << endl;

}

/**

Computes the gradient given \f$ t_ik \f$ .


 @param[in] tik : fuzzy classification.
 @param[out] Q : Gradient vector.

Abstract. Model-dependent.

*/


void MRF::Gradient(vector<double> const &  tik, vector<double> & Q){
  
  cerr << "Warning! MRF::Gradient(vector<double> const &  tik, vector<double> & Q) is virtual." << endl;

}


  
/** 

Number of free parameters.

@return the degrees of freedom of the MRF model.

Abstract. Model-dependent.

*/

uint MRF::Degrees_Of_Freedom(){

  cerr << "MRF::Degrees_Of_Freedom() -- Don't know how to do this ... programming error ?" << endl;
  
  return(0);

}


/**
Read the model parameters from the stream \b is.

*/

void MRF::Read_Params(istream & is){
  cout << "Warning! MRF::Read_Params(istream & is) is virtual."  << endl;
}

/**

Write the model parameters to the stream os.

*/

void MRF::Write_Params(ostream & os){
  cout << "Warning! MRF::Write_Params(ostream & os) is virtual."  << endl;

}

/**
   
Print some info on the model.

*/

void MRF::Info(){
  cout << "Warning! MRF::Info() is virtual."  << endl;
}


