#include <NCEM_Algo.hpp>
#include <Stat.hpp>

NCEM_Algorithm::NCEM_Algorithm(vector<double> bet, IID_Mixture *mod, Spatial_Data *spatd){
  
  Model=mod;
  SpatData=spatd;
  uint K=mod->Get_K();
  Beta.resize(K*K);
  if(bet.size()==(K*K))
    for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=bet[idx]; 
  else{
    cerr << "NCEM_Algorithm::NCEM_Algorithm. Size error." << endl;
		for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=0.0;
    for(uint idx=0 ; idx < K ; idx++) Beta[idx+idx*K]=1.0; 
  }
  Use_Default_Stopping_Rule();
  uint N=Get_N();
  logdens.clear();
	logdens.resize(N*K);
}


NCEM_Algorithm::NCEM_Algorithm(double bet, IID_Mixture *mod, Spatial_Data *spatd){
  
  Model=mod;
  SpatData=spatd;
  uint K=mod->Get_K();
  Beta.resize(K*K);
  for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=0.0; 
  for(uint idx=0 ; idx < K ; idx++) Beta[idx+idx*K]=bet; 
  Use_Default_Stopping_Rule();
  uint N=Get_N();
  logdens.clear();
	logdens.resize(N*K);
}


NCEM_Algorithm::NCEM_Algorithm(IID_Mixture *mod, Spatial_Data *spatd){
  Model=mod;
  SpatData=spatd;
  uint K=Model->Get_K();
  Beta.resize(K*K);
  for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=0.0; 
  for(uint idx=0 ; idx < K ; idx++) Beta[idx+idx*K]=1.0;   
  Use_Default_Stopping_Rule();
  uint N=Get_N();
  logdens.clear();
	logdens.resize(N*K);
}


NCEM_Algorithm::~NCEM_Algorithm(){

}


void NCEM_Algorithm::Set_Beta(vector<double> bet){
  uint K=Model->Get_K();
  Beta.resize(K*K);
  if(bet.size()==(K*K))
    for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=bet[idx]; 
  else{
    cerr << "NCEM_Algorithm::SetBeta. Size error." << endl;
    for(uint idx=0 ; idx < K ; idx++) Beta[idx+idx*K]=1.0; 
  }
  
}


void NCEM_Algorithm::Compute_fuzzy(uint i, vector<double> & probs){
	uint K=Model->Get_K();
	probs.clear();
	probs.resize(K);
	
	vector< Weighted_Neighbor * >  ineighs;
	SpatData->GetNeighbors(i,ineighs);
	uint nv=ineighs.size();
	
	double max=-1.0;
	uint kmax=0;
	vector<double> Pi;
	Model->Get_Pi(Pi);
	for(uint k=0 ; k < K ; k++){
		double temp=0.0;
		for(uint l=0 ; l < K ; l++){
			for(uint j=0;j<nv;j++){
			uint neidx=ineighs[j]->Index;
			temp+=Beta[k*K+l]*fuzzy[neidx*K+l];
		}
		double curr=Pi[k]* Exp (logdens[i*K+k] + temp);
		if (curr>max) {kmax=k;max=curr;}
	}
	for (uint k=0;k<K;k++) probs[k]=0.0;    
	probs[kmax]=1.0;
	ineighs.clear();
	}

}

