#include <NEM_Algo.hpp>
#include <Stat.hpp>
#include <Utils.hpp>

NEM_Algorithm::NEM_Algorithm(vector<double> bet, IID_Mixture *mod, Spatial_Data *spatd){
  
  Model=mod;
  SpatData=spatd;
  uint K=mod->Get_K();
  Beta.resize(K*K);
  if(bet.size()==(K*K))
    for(uint idx=1 ; idx < (K*K) ; idx++) Beta[idx]=bet[idx]; 
  else{
    cerr << "NEM_Algorithm::NEM_Algorithm. Size error." << endl;
    for(uint idx=1 ; idx < K ; idx++) Beta[idx+idx*K]=1.0; 
  }
  Use_Default_Stopping_Rule();
  uint N=Get_N();
  logdens.clear();
	logdens.resize(N*K);

}

NEM_Algorithm::NEM_Algorithm(double bet, IID_Mixture *mod, Spatial_Data *spatd){
  
  Model=mod;
  SpatData=spatd;
  uint K=mod->Get_K();
  Beta.resize(K*K);
  for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=0.0; 
  for(uint idx=0 ; idx < K ; idx++) Beta[idx+idx*K]=bet; 
  Use_Default_Stopping_Rule();
  uint N=Get_N();
  logdens.clear();
	logdens.resize(N*K);
}

NEM_Algorithm::NEM_Algorithm(IID_Mixture *mod, Spatial_Data *spatd){
  Model=mod;
  SpatData=spatd;
  uint K=Model->Get_K();
  Beta.resize(K*K);
  for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=0.0;
  for(uint idx=0 ; idx < K ; idx++) Beta[idx+idx*K]=1.0;   
  Use_Default_Stopping_Rule();
  uint N=Get_N();
  logdens.clear();
	logdens.resize(N*K);
}


NEM_Algorithm::~NEM_Algorithm(){

}

void NEM_Algorithm::Set_Beta(vector<double> bet){
  uint K=Model->Get_K();
  Beta.resize(K*K);
  if(bet.size()==(K*K))
    for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=bet[idx]; 
  else{
    cerr << "NEM_Algorithm::Set_Beta. Size error." << endl;
    for(uint idx=0 ; idx < (K*K) ; idx++) Beta[idx]=0.0;
    for(uint idx=0 ; idx < K ; idx++) Beta[idx+idx*K]=1.0; 
  }
  
}

void NEM_Algorithm::Compute_fuzzy(uint i, vector<double> & probs){
	uint K=Model->Get_K();
	probs.clear();
	probs.resize(K);

	vector< Weighted_Neighbor * >  ineighs;
	SpatData->GetNeighbors(i,ineighs);
	uint nv=ineighs.size();

	vector<double> Pi;
	Model->Get_Pi(Pi);
	vector<double> temp(K,0);
	double max=0.0;
	for(uint k=0 ; k < K ; k++){
		for(uint l=0 ; l < K ; l++){
			for(uint j=0;j<nv;j++){
			uint neidx=ineighs[j]->Index;
			temp[k]+=Beta[k*K+l]*fuzzy[neidx*K+l];
			}
		}
		if (temp[k]+logdens[i*K+k]>max) max=temp[k]+logdens[i*K+k];
	}
	
	double sum=0.0;	
	for(uint k=0 ; k < K ; k++){
		probs[k]=Pi[k]* Exp (logdens[i*K+k] + temp[k] - max);
		sum += probs[k];
	}    
	
	for(uint k=0 ; k < K ; k++) probs[k] /= sum;
	ineighs.clear();
	temp.clear();
	
	//WriteVector<double>(cerr,probs);

}
