#include <Seg_Algorithm.hpp>

uint Seg_Algorithm::Get_N(){
	cerr <<  "Warning! Seg_Algorithm::Get_N() is virtual." << endl;
	return 0;
}

uint Seg_Algorithm::Get_K(){
	cerr <<  "Warning! Seg_Algorithm::Get_K() is virtual." << endl;
	return 0;
}


Seg_Algorithm::~Seg_Algorithm(){

  Crit_Values.clear();
  old_fuzzy.clear();
  old_map_labs.clear();
  fuzzy.clear();

}

void Seg_Algorithm::Init_Labels(vector<uint> const &  labs){
  cerr << "Warning! Seg_Algorithm::Init_Labels() is virtual." << endl;
}

void Seg_Algorithm::Init_Random(){
  cerr << "Warning! Seg_Algorithm::Init_Random() is virtual." << endl;
}

/// Test if the algorithm converged ...
bool Seg_Algorithm::HasConverged(){
  bool b=false;
  // Number of iterations
  if(Criterion==ITER_CRIT){
    if(Crit_Values[Criterion] >= Tol) b=true;
  }
  // otherwise
  else{
    if(Crit_Values[Criterion] <= Tol) b=true;
  }
  // At least one iteration.
  if(Crit_Values[ITER_CRIT]==0)  b=false;
  return(b);
}

void Seg_Algorithm::Get_Stopping_Crit(vector<double> &crit){
	crit.clear();
	crit.resize(Crit_Values.size());
	for(uint c=0;c<Crit_Values.size();c++) crit[c]=Crit_Values[c];
}

void Seg_Algorithm::Info(){
  cerr << "Warning! Seg_Algorithm::Info() is virtual." << endl;
}

void Seg_Algorithm::Compute_MAP_Labels(vector<uint> & Labels){
	cerr << "Warning! Seg_Algorithm::Compute_MAP_Labels(vector<uint> & Labels) is virtual." << endl;
}

void Seg_Algorithm::Compute_fuzzy(uint i,vector<double> &probs){
  cerr << "Warning! Seg_Algorithm::Compute_fuzzy(uint i,vector<double> &probs) is virtual." << endl;
}


/// Run for a number of iterations ...
void Seg_Algorithm::Run(uint nbiter){
  cerr << "Warning! Seg_Algorithm::Run(uint nbiter) is virtual." << endl;
}

void Seg_Algorithm::E_Step(){
  cerr << "Warning! Seg_Algorithm::E_Step() is virtual." << endl;
}

void Seg_Algorithm::M_Step(){
	cerr << "Warning! Seg_Algorithm::M_Step() is virtual." << endl;
}

/// Run until the convergence test is ok ...
void Seg_Algorithm::RunUntilConvergence(){
  
  bool cvg=false;
  uint iter=0;
  while (!cvg){
    Run(1);
    Update_Stopping_Rules();
    cvg=HasConverged();
    iter++;
    //Info();
    Display_Stopping_Rule();
  }
  cerr << "Converged after " << iter << " iterations." << endl;
}


/// Completed likelihood criterion
void Seg_Algorithm::Compute_CLCrit(){
}
/// Fuzzy classification criterion 
void Seg_Algorithm::Compute_FuzzyCrit(){
}
/// Hard classification criterion
void Seg_Algorithm::Compute_HardCrit(){
}


void Seg_Algorithm::Update_Stopping_Rules(){
  if(Crit_Values.size()==0){
    Crit_Values.resize(MAX_CRIT);
    Crit_Values[ITER_CRIT]=0;
    Compute_CLCrit();
    Compute_FuzzyCrit();
    Compute_HardCrit();    
  }
  else{
    Crit_Values[ITER_CRIT] += 1.0;
    Compute_CLCrit();
    Compute_FuzzyCrit();
    Compute_HardCrit();
  }
}

void Seg_Algorithm::Use_Default_Stopping_Rule(){
  Set_Stopping_Rule(ITER_CRIT, 1.0);
}

void Seg_Algorithm::Set_Stopping_Rule(uint crit, double tol){
  if(crit < MAX_CRIT){
    Criterion = crit;
    Tol=tol;
  }
  else{
    cerr << " Seg_Algorithm::Set_Stopping_Rule().\n Stopping rule specification error." << endl;
  }
}


void Seg_Algorithm::Display_Stopping_Rule(){
  cerr << "Criterion : " << Criterion << "\t Tolerance : " << Tol <<  endl;
  cerr << "Current values : " ;
  for(uint c=0;c<Crit_Values.size();c++) cerr << Crit_Values[c] << " " ;
  cerr << endl;
}

