#include <Seg_Model.hpp>
#include <Stat.hpp>

/**
   Destructor.

Abstract. Model-dependent.

*/

Seg_Model::~Seg_Model(){
	Distrib.clear();
};

/**
 Get the dimension
*/

uint Seg_Model::Get_D(){
  return(Distrib[0]->Get_D());
}

/**
Get the number of classes/colors.

@return The number of classes/colors in the MRF.

*/

uint Seg_Model::Get_K(){
  return(Distrib.size());
}


/**
Compute the  BIC -- Bayesian Information Criterion.

  @param[in] spatd : Spatial data (observations).

  @return the value of the BIC.

Abstract. Model-dependent.


*/

double Seg_Model::BIC(Spatial_Data *spatd){
  cerr << "Warning. Seg_Model::BIC() is not defined." << endl;
  return(0.0);
}

/**
Compute the ICL --  Integrated Completed Likelihood

  @param[in] spatd : Spatial data (observations).

  @return the value of the ICL.

Abstract. Model-dependent.


*/

double Seg_Model::ICL(Spatial_Data *spatd){
  
  cerr << "Warning. Seg_Model::ICL() is not defined." << endl;
  return(0.0);
}

/**
   Comptue the Log-Likelihood


  @param[in] spatd : Spatial data (observations).


  @return the value of the log-likelihood.

Abstract. Model-dependent.

*/


double Seg_Model::LogLikelihood(Spatial_Data *spatd){
  cerr << "Warning. Seg_Model::Likelihood() is not defined." << endl;
  return(0.0);
}


/**

 Compute the Completed likelihood.

 @param[in] spatd : Spatial data (observations). 
 @param[in] tik : fuzzy classifications

 @return the value of the completed likelihhood.

Abstract. Model-dependent.

*/


double Seg_Model::Completed_LogLikelihood(Spatial_Data*, vector<double> tik){
  cerr << "Warning. Seg_Model::Completed_Likelihood() is not defined." << endl;
  return(0.0);
}

/** 
    
Compute log density at X in the  class k, i.e. , log f(X | mu_k, sigma_k)

*/

double Seg_Model::LogDensity(vector<double> const &  X, uint k){
  return(Distrib[k]->LogDensity(X));
}  

/** 
    
Compute density at X in the  class k, i.e. , f(X | mu_k, sigma_k)

*/

double Seg_Model::Density(vector<double> const &  X, uint k){
	return(Distrib[k]->Density(X));
}  

void Seg_Model::Estimate_Distributions(Spatial_Data *spatd, vector<double>  const &  tik){
	uint K=Get_K();
	uint N=spatd->Get_N();

  for(uint k=0 ; k < K ; k++){
    vector<double> freqs(N);
    for(uint i=0 ; i < N ; i++) freqs[i]=tik[i*K+k];
    Distrib[k]->Estimate(spatd,freqs);
    freqs.clear();
  }

}

/** 

Number of free parameters.

@return the degrees of freedom of the MRF model.

Abstract. Model-dependent.

*/

uint Seg_Model::Degrees_Of_Freedom(){
  cerr << "Warning. Seg_Model::Degrees_Of_Freedom() is not defined." << endl;
  return(0);
}

/**
   
Print some info on the model.

Abstract. Model-dependent.

*/


void Seg_Model::Info(){
    cerr << "Warning. Seg_Model is abstract." << endl;
};

void Seg_Model::Estimate(Spatial_Data *spatd, vector<double>  const &  tik){
    cerr << "Warning. Seg_Model is abstract." << endl;
};

