#include <Spatial_Data.hpp>
#include <External_Potts.hpp>
#include <Generic_Potts.hpp>
#include <Potts.hpp>
#include <Utils.hpp>
#include <Stat.hpp>
#include <LinAlg.hpp>
#include <Diag_Normal.hpp>
#include <Normal.hpp>
#include <HMRF.hpp>
#include <IID_Mixture.hpp>
#include <Simulated_Field_EM.hpp>
#include <Modal_Field_EM.hpp>
#include <Mean_Field_EM.hpp>
#include <EM_Algo.hpp>
#include <NEM_Algo.hpp>
#include <NCEM_Algo.hpp>
#include <CEM_Algo.hpp>
#include <ICM_Algo.hpp>
#include <Neighborhood_System.hpp>

int main(int argc, char **argv){
	//1er argument a donner : le nom du fichier de donnees...
	//2eme argument a donner en lancant la commande : nombre maximum de clusters
	//3eme argument : dimension des donnees
	//4eme argument : nb d'iterations maximal
	uint K=fromString<uint>(argv[2]);
	uint dim=fromString<uint>(argv[3]);
	uint nb_iter=fromString<uint>(argv[4]);	
	
	//LECTURE DES DONNEES dans "monfichier.str, .dat et .nei"
	Spatial_Data *dat = new Spatial_Data();
	dat->ReadFromFile(argv[1]);
	
	//affichage d'un des caracteristiques des donnees ; ici le nb d'individus
	//cout << "N="<< dat->Get_N() << endl;
		
	//creation des INFOS DE VOISINAGE a partir du fichier de donnees lues
	Neighborhood_System *ns=new Neighborhood_System();
	ns=dat->Get_NS();
	
	//les algos tourneront pr des NB DE CLASSES de 2 a K 
	for (uint k=6;k<K+1;k++) {
/*		cout << "-->>nb de classes=" << k << " et algo=SF_diag" << endl;
		//on declare un vecteur qui contiendra les densites (ici normales, variances diagonales) des composantes du melange
	        vector <Diag_Normal*> vecdiag_sf(k); 
		//on remplit le vecteur composante par composante (autant que de classes)
		for (uint l=0;l<k;l++){
			vecdiag_sf[l]=new Diag_Normal(dim);
		}
		
		//on declare le modele de voisinage (ici potts generic = beta qui donne l'interaction entre les classes)
		Generic_Potts *genpotts_sfdiag=new Generic_Potts(ns,k);
		
		//on cree le champ de markov cache grace aux modeles de voisinage et sur les donnees
		HMRF *hmrf_sfdiag=new HMRF(genpotts_sfdiag,vecdiag_sf);
		
		//on declare l'ALGO (SF ici) a partir du chp de markov cache et des donnees
		Simulated_Field_EM *sfdiag=new Simulated_Field_EM(hmrf_sfdiag,dat);
		
		//l'initialisation de l'algo. Ici on assigne les classes "au hasard"
		//sfdiag->Init_Random();
		//et initialisation par k-means
		sfdiag->Init_Random();
		
		//critere d'arret de l'algorithme
		//Ici qd ttes les probas d'appartenance aux classes bougent de moins de 0.01
		//sfdiag->Set_Stopping_Rule(2,0.01);
		//Ou la on limite le nombre d'iterations
		sfdiag->Set_Stopping_Rule(0,nb_iter);
		
		//On lance l'algo en respectant le critere d'arret
		sfdiag->RunUntilConvergence();
		
		//vector<double> crit; on essaye de stopper l'algo sur 2 criteres simultannes
		//for (uint iter=1;iter<=500;iter++){
		//	sfdiag->Run(1);
		//	sfdiag->Get_Stopping_Crit(crit);
		//	if (crit[2]<=0.01) exit;
		//}
		
		//LES SORTIES...
		
		//cls=le vecteur des classes
		vector<uint> cls_sfdiag;
		sfdiag->Compute_MAP_Labels(cls_sfdiag);
		string cls_name1="lesclassesSFdiag_";
		cls_name1+=argv[1];
		ofstream file_cls1(cls_name1.c_str(),ofstream::out | ofstream::app);//ofstream file_cls("lesclassesSFdiag_chu.txt");
		//on ecrit le nombre de classes (pr y voir clair)
		file_cls1 << " k= " << k << endl;
		//on ecrit les cls ds 1 unique fichier "lesclassesSFdiag_chu.txt" donnant le nb d'objet (N) suivi des N assignation de classes sur une meme ligne. Autant de ligne que de nb de classes testees (K)
		WriteVector<uint>(file_cls1,cls_sfdiag);
		
		//idem mu=le vecteur des moyennes et sig=la "matrice" de covariance (sur 1 ligne)
		string mu_name1="muSFdiag_";
		mu_name1+=argv[1];
		ofstream file_mu1(mu_name1.c_str(),ofstream::out | ofstream::app);
		//string sig_name1="sigmaSFdiag_";
		// sig_name1+=argv[1];
		//ofstream file_sig1(sig_name1.c_str(),ofstream::out | ofstream::app);
		//on ecrit aussi le nombre de classes avant les parametres...
		file_mu1 << "-->>k= " << k << endl;
		for (uint l=0;l<k;l++){
			vector<double> mu1;
			//vector<double> sigma1;	
			vecdiag_sf[l]->Get_Mu(mu1);
			//vecdiag_sf[l]->Get_Sigma(sigma1);
			WriteVector<double>(file_mu1,mu1);
			//WriteVector<double>(file_sig1,sigma1);
		}
		file_mu1 << endl;
		
		//calcul du BIC sur les donnees (a partir du modele sfdiag)
		double BIC_sfdiag=hmrf_sfdiag->BIC(dat);
		file_cls1 << BIC_sfdiag << endl;
		file_cls1 << endl;
		
		//...IDEM SUR LES AUTRES ALGOS
*/		
		//algo EM diag normal
		
		cout << "-->>nb de classes=" << k << " et algo=EM_diag" << endl;
		vector <Diag_Normal*> vecdiag_em(k); 
		for (uint l=0;l<k;l++){
			vecdiag_em[l]=new Diag_Normal(dim);
		}
		IID_Mixture *mixt_emdiag=new IID_Mixture(k,vecdiag_em);
		
		EM_Algorithm *emdiag=new EM_Algorithm(mixt_emdiag,dat);
		
		emdiag->Init_Random();
		emdiag->Set_Stopping_Rule(0,nb_iter);
		emdiag->RunUntilConvergence();
		
		vector<uint> cls_emdiag;
		emdiag->Compute_MAP_Labels(cls_emdiag);
		string cls_name2="lesclassesEMdiag_";
		cls_name2+=argv[1];
		ofstream file_cls2(cls_name2.c_str(),ofstream::out | ofstream::app);
		file_cls2 << " k= " << k << endl;
		WriteVector<uint>(file_cls2,cls_emdiag);
		
		string mu_name2="muEMdiag_";
		mu_name2+=argv[1];
		ofstream file_mu2(mu_name2.c_str(),ofstream::out | ofstream::app);
		//string sig_name2="sigmaEMdiag_chu.txt";
		//sig_name2+=argv[1];
		//ofstream file_sig2(sig_name2.c_str(),ofstream::out | ofstream::app);
		file_mu2 << " k= " << k << endl;
		for (uint l=0;l<k;l++){
			vector<double> mu2;
			//vector<double> sigma2;	
			vecdiag_em[l]->Get_Mu(mu2);
			//vecdiag_em[l]->Get_Sigma(sigma2);
			WriteVector<double>(file_mu2,mu2);
			//WriteVector<double>(file_sig2,sigma2);
		}
		file_mu2 << endl;
		
		double BIC_emdiag=mixt_emdiag->BIC(dat);
		file_cls2 << BIC_emdiag << endl;
		file_cls2 << endl;
/*		
		//algo EM normal
		
		cout << "-->>nb de classes=" << k << " et algo=EM_gal" << endl;
		vector <Normal*> vec_em(k); 
		for (uint l=0;l<k;l++){
			vec_em[l]=new Normal(dim);
		}
		IID_Mixture *mixt_em=new IID_Mixture(k,vec_em);
		
		EM_Algorithm *em=new EM_Algorithm(mixt_em,dat);

		em->Init_Random();
		em->Set_Stopping_Rule(0,nb_iter);
		em->RunUntilConvergence();
		
		
		vector<uint> cls_em;
		em->Compute_MAP_Labels(cls_em);
		string cls_name3="lesclassesEM_";
		cls_name3+=argv[1];
		ofstream file_cls3(cls_name3.c_str(),ofstream::out | ofstream::app);
		file_cls3 << " k= " << k << endl;
		WriteVector<uint>(file_cls3,cls_em);
		
		string mu_name3="muEM_";
		mu_name3+=argv[1];
		ofstream file_mu3(mu_name3.c_str(),ofstream::out | ofstream::app);
		//string sig_name3="sigmaEM_";
		//sig_name3+=argv[1];
		//ofstream file_sig3(sig_name3.c_str(),ofstream::out | ofstream::app);
		file_mu3 << " k= " << k << endl;
		for (uint l=0;l<k;l++){
			vector<double> mu3;
			//vector<double> sigma3;	
			vec_em[l]->Get_Mu(mu3);
			//vec_em[l]->Get_Sigma(sigma3);
			WriteVector<double>(file_mu3,mu3);
			//WriteVector<double>(file_sig3,sigma3);
		}
		file_mu3 << endl;
		
		double BIC_em=mixt_em->BIC(dat);
		file_cls3 << BIC_em << endl;
		file_cls3 << endl;
		
		//algo EM reduc dim
		
		cout << "-->>nb de classes=" << k << " et algo=EM_hd" << endl;
		vector <HighDim_Normal*> vec_emhd(k); 
		for (uint l=0;l<k;l++){
			vec_emhd[l]=new HighDim_Normal(dim);
		}
		IID_Mixture *mixt_emhd=new IID_Mixture(k,vec_emhd);
		
		EM_Algorithm *emhd=new EM_Algorithm(mixt_emhd,dat);
		
		emhd->Init_KMeans();
		emhd->Set_Stopping_Rule(0,nb_iter);
		emhd->RunUntilConvergence();
		
		vector<uint> cls_emhd;
		emhd->Compute_MAP_Labels(cls_emhd);
		string cls_name4="lesclassesEMhd_";
		cls_name4+=argv[1];
		ofstream file_cls4(cls_name4.c_str(),ofstream::out | ofstream::app);
		file_cls4 << " k= " << k << endl;
		WriteVector<uint>(file_cls4,cls_emhd);
		
		string mu_name4="muEMhd_";
		mu_name4+=argv[1];
		ofstream file_mu4(mu_name4.c_str(),ofstream::out | ofstream::app);
		
		file_mu4 << " k= " << k << endl;
		for (uint l=0;l<k;l++){
			vector<double> mu4;
		
			vec_emhd[l]->Get_Mu(mu4);
		
			WriteVector<double>(file_mu4,mu4);
		
		}
		file_mu4 << endl;
		
		double BIC_emrd=mixt_emhd->BIC(dat);
		file_cls4 << BIC_emrd << endl;
		file_cls4 << endl;
*/		
		//algo MF diag normal
		
		cout << "-->>nb de classes=" << k << " et algo=MF_diag" << endl;
		vector <Diag_Normal*> vecdiag_mf(k); 
		for (uint l=0;l<k;l++){
			vecdiag_mf[l]=new Diag_Normal(dim);
		}
		
		Generic_Potts *genpotts_mfdiag=new Generic_Potts(ns,k);
		
		HMRF *hmrf_mfdiag=new HMRF(genpotts_mfdiag,vecdiag_mf);
		
		Mean_Field_EM *mfdiag=new Mean_Field_EM(hmrf_mfdiag,dat);
		
		mfdiag->Init_KMeans();
		mfdiag->Set_Stopping_Rule(0,nb_iter);
		mfdiag->RunUntilConvergence();
		
		vector<uint> cls_mfdiag;
		mfdiag->Compute_MAP_Labels(cls_mfdiag);
		string cls_name5="lesclassesMFdiag_";
		cls_name5+=argv[1];
		ofstream file_cls5(cls_name5.c_str(),ofstream::out | ofstream::app);
		file_cls5 << " k= " << k << endl;
		WriteVector<uint>(file_cls5,cls_mfdiag);
		
		string mu_name5="muMFdiag_";
		mu_name5+=argv[1];
		ofstream file_mu5(mu_name5.c_str(),ofstream::out | ofstream::app);
		//string sig_name5="sigmaMFdiag_";
		//sig_name5+=argv[1];
		//ofstream file_sig5(sig_name5.c_str(),ofstream::out | ofstream::app);
		file_mu5 << " k= " << k << endl;
		for (uint l=0;l<k;l++){
			vector<double> mu5;
		//	vector<double> sigma5;	
			vecdiag_mf[l]->Get_Mu(mu5);
		//	vecdiag_mf[l]->Get_Sigma(sigma5);
			WriteVector<double>(file_mu5,mu5);
		//	WriteVector<double>(file_sig5,sigma5);
		}
		file_mu5 << endl;
		
		double BIC_mfdiag=hmrf_mfdiag->BIC(dat);
		file_cls5 << BIC_mfdiag << endl;
		file_cls5 << endl;
/*		
		//algo MF "reduc dim"
		
		cout << "-->>nb de classes=" << k << " et algo=MF_hd" << endl;
		vector <HighDim_Normal*> vec_mfhd(k); 
		for (uint l=0;l<k;l++){
			vec_mfhd[l]=new HighDim_Normal(dim);
		}
		
		Generic_Potts *genpotts_mfhd=new Generic_Potts(ns,k);
		
		HMRF *hmrf_mfhd=new HMRF(genpotts_mfhd,vec_mfhd);
		
		Mean_Field_EM *mfhd=new Mean_Field_EM(hmrf_mfhd,dat);
		
		mfhd->Init_KMeans();
		mfhd->Set_Stopping_Rule(0,nb_iter);
		mfhd->RunUntilConvergence();
		
		vector<uint> cls_mfhd;
		mfhd->Compute_MAP_Labels(cls_mfhd);
		string cls_name6="lesclassesMFhd_";
		cls_name6+=argv[1];
		ofstream file_cls6(cls_name6.c_str(),ofstream::out | ofstream::app);
		file_cls6 << " k= " << k << endl;
		WriteVector<uint>(file_cls6,cls_mfhd);
		
		string mu_name6="muMFhd_";
		mu_name6+=argv[1];
		ofstream file_mu6(mu_name6.c_str(),ofstream::out | ofstream::app);
		
		file_mu6 << " k= " << k << endl;
		for (uint l=0;l<k;l++){
			vector<double> mu6;
		
			vec_mfhd[l]->Get_Mu(mu6);
		
			WriteVector<double>(file_mu6,mu6);
		
		}
		file_mu6 << endl;
		
		double BIC_mfrd=hmrf_mfhd->BIC(dat);
		file_cls6 << BIC_mfrd << endl;
		file_cls6 << endl;
		
		//algo SF normal
		
		cout << "-->>nb de classes=" << k << " et algo=SF_gal" << endl;
		vector <Normal*> vec_sf(k); 
		for (uint l=0;l<k;l++){
			vec_sf[l]=new Normal(dim);
		}
		
		Generic_Potts *genpotts_sf=new Generic_Potts(ns,k);
		
		HMRF *hmrf_sf=new HMRF(genpotts_sf,vec_sf);
		
		Simulated_Field_EM *sf=new Simulated_Field_EM(hmrf_sf,dat);
		
		sf->Init_KMeans();
		sf->Set_Stopping_Rule(0,nb_iter);
		sf->RunUntilConvergence();
		
		vector<uint> cls_sf;
		sf->Compute_MAP_Labels(cls_sf);
		string cls_name7="lesclassesSF_";
		cls_name7+=argv[1];
		ofstream file_cls7(cls_name7.c_str(),ofstream::out | ofstream::app);
		file_cls7 << " k= " << k << endl;
		WriteVector<uint>(file_cls7,cls_sf);
		
		string mu_name7="muSF_";
		mu_name7+=argv[1];
		ofstream file_mu7(mu_name7.c_str(),ofstream::out | ofstream::app);
		//string sig_name7="sigmaSF_";
		//sig_name7+=argv[1];
		//ofstream file_sig7(sig_name7.c_str(),ofstream::out | ofstream::app);
		file_mu7 << " k= " << k << endl;
		for (uint l=0;l<k;l++){
			vector<double> mu7;
		//	vector<double> sigma7;	
			vec_sf[l]->Get_Mu(mu7);
		//	vec_sf[l]->Get_Sigma(sigma7);
			WriteVector<double>(file_mu7,mu7);
		//	WriteVector<double>(file_sig7,sigma7);
		}
		file_mu7 << endl;
		
		double BIC_sf=hmrf_sf->BIC(dat);
		file_cls7 << BIC_sf << endl;
		file_cls7 << endl;
		
		//algo SF "reduc dim"
		
		cout << "-->>nb de classes=" << k << " et algo=SF_hd" << endl;
		vector <HighDim_Normal*> vec_sfhd(k); 
		for (uint l=0;l<k;l++){
			vec_sfhd[l]=new HighDim_Normal(dim);
		}
		
		Generic_Potts *genpotts_sfhd=new Generic_Potts(ns,k);
		
		HMRF *hmrf_sfhd=new HMRF(genpotts_sfhd,vec_sfhd);
		
		Simulated_Field_EM *sfhd=new Simulated_Field_EM(hmrf_sfhd,dat);
		
		sfhd->Init_KMeans();
		sfhd->Set_Stopping_Rule(0,nb_iter);
		sfhd->RunUntilConvergence();
		
		vector<uint> cls_sfhd;
		sfhd->Compute_MAP_Labels(cls_sfhd);
		string cls_name8="lesclassesSFhd_";
		cls_name8+=argv[1];
		ofstream file_cls8(cls_name8.c_str(),ofstream::out | ofstream::app);
		file_cls8 << " k= " << k << endl;
		WriteVector<uint>(file_cls8,cls_sfhd);
		
		string mu_name8="muSFhd_";
		mu_name8+=argv[1];
		ofstream file_mu8(mu_name8.c_str(),ofstream::out | ofstream::app);
		
		file_mu8 << " k= " << k << endl;
		for (uint l=0;l<k;l++){
			vector<double> mu8;
			
			vec_sfhd[l]->Get_Mu(mu8);
		
			WriteVector<double>(file_mu8,mu8);
		
		}
		file_mu8 << endl;
		
		double BIC_sfrd=hmrf_sfhd->BIC(dat);
		file_cls8 << BIC_sfrd << endl;
		file_cls8 << endl;
*/
	}
}
