#include <Spatial_Data.hpp>
#include <External_Potts.hpp>
#include <Potts.hpp>
#include <Utils.hpp>
#include <Stat.hpp>
#include <LinAlg.hpp>
#include <Diag_Normal.hpp>
#include <Normal.hpp>
#include <HMRF.hpp>
#include <IID_Mixture.hpp>
#include <Simulated_Field_EM.hpp>
#include <Modal_Field_EM.hpp>
#include <Mean_Field_EM.hpp>
#include <EM_Algo.hpp>
#include <NEM_Algo.hpp>
#include <NCEM_Algo.hpp>
#include <CEM_Algo.hpp>
#include <ICM_Algo.hpp>
#include <Neighborhood_System.hpp>

int main(int argc, char **argv){

  if((argc>7) || (argc<2)){
    cout << "usage :  " << argv[0] << " K Nl Nc nbgibbs dim beta" << endl;
  }
  else{
    srand(time(0));
    srand48(time(0));

    uint K=fromString<uint>(argv[1]);
    uint Nl=fromString<uint>(argv[2]);
    uint Nc=fromString<uint>(argv[3]);   
    uint nbiter=fromString<uint>(argv[4]);
    uint D=fromString<uint>(argv[5]);
    double beta=fromString<double>(argv[6]);

    Image_Local_Neighborhood *ns=new Image_Local_Neighborhood(Nl,Nc);
    ns->Info();
    
    uint N=ns->Get_N();
    cout << "N=" << N << " " << "D=" << D << " " << endl; 

    vector<double> pot(K);
    for (uint k=0;k<K;k++){
      pot[k]=(double) k/K;
    }
    WriteVector<double>(cout,pot);
    External_Potts *mrf=new External_Potts(ns,K,pot,beta);

    vector<Normal*> gaussians(K);
    for (uint k=0;k<K;k++){
      gaussians[k]=new Normal(D);
      vector<double> Mu(D);
      vector<double> Sigma(D);
      for (uint d=0;d<D;d++){
				Mu[d]=k+d;
				Sigma[d]=0.25*(d+1);
      }
      gaussians[k]->Set_Mu(Mu);
      gaussians[k]->Set_Sigma(Sigma); 
      WriteVector<double>(cout,Mu); 
      WriteVector<double>(cout,Sigma);
      Mu.clear();
      Sigma.clear();
      
    }
 
    HMRF *hmrf=new HMRF(mrf,gaussians);
    Data *dat=new Data(N,D);
    

    hmrf->Simulate(dat,nbiter);
    vector<uint> truelabs;
    dat->GetLabels(truelabs);

    Spatial_Data *spatd=new Spatial_Data(dat,ns);

    // EM 
    vector<Normal*> gaussians1(K);
    for (uint k=0;k<K;k++){
      gaussians1[k]=new Normal(D);
    }
    IID_Mixture *mod1=new IID_Mixture(K,gaussians1);
    EM_Algorithm *algo1=new EM_Algorithm(mod1,spatd);

    // NEM beta=1.0
    vector<Normal*> gaussians2(K);
    for (uint k=0;k<K;k++){
      gaussians2[k]=new Normal(D);
    }
    IID_Mixture *mod2=new IID_Mixture(K,gaussians2);
    NEM_Algorithm *algo2=new NEM_Algorithm(5.0,mod2,spatd);

    // CEM
    vector<Normal*> gaussians3(K);
    for (uint k=0;k<K;k++){
      gaussians3[k]=new Normal(D);
    }
    IID_Mixture *mod3=new IID_Mixture(K,gaussians3);
    CEM_Algorithm *algo3=new CEM_Algorithm(mod3,spatd);
    

    // NCEM beta=1.0
    vector<Normal*> gaussians4(K);
    for (uint k=0;k<K;k++){
      gaussians4[k]=new Normal(D);
    }
    IID_Mixture *mod4=new IID_Mixture(K,gaussians4);
    NCEM_Algorithm *algo4=new NCEM_Algorithm(1.0,mod4,spatd);

    // ICM
    External_Potts *mrf5=new External_Potts(ns,K);
    vector<Normal*> gaussians5(K);
     for (uint k=0;k<K;k++){
       gaussians5[k]=new Normal(D);
     }
     HMRF *mod5=new HMRF(mrf5,gaussians5);
     ICM_Algorithm *algo5 = new ICM_Algorithm(mod5,spatd);

    // Mean field
    External_Potts *mrf6=new External_Potts(ns,K);
    vector<Normal*> gaussians6(K);
    for (uint k=0;k<K;k++){
      gaussians6[k]=new Normal(D);
    }
    HMRF *mod6=new HMRF(mrf6,gaussians6);
    Mean_Field_EM *algo6 = new Mean_Field_EM(mod6,spatd,1);
    
    // Modal field
    External_Potts *mrf7=new External_Potts(ns,K);
    vector<Normal*> gaussians7(K);
    for (uint k=0;k<K;k++){
      gaussians7[k]=new Normal(D);
    }
    HMRF *mod7=new HMRF(mrf7,gaussians7);
    Modal_Field_EM *algo7 = new Modal_Field_EM(mod7,spatd,1);

    // Simulated field
    External_Potts *mrf8=new External_Potts(ns,K);
    vector<Normal*> gaussians8(K);
    for (uint k=0;k<K;k++){
      gaussians8[k]=new Normal(D);
    }
    HMRF *mod8=new HMRF(mrf8,gaussians8);
    Simulated_Field_EM *algo8 = new Simulated_Field_EM(mod8,spatd,1,10);

    algo1->Init_KMeans();
    algo2->Init_KMeans();
    algo3->Init_KMeans();
    algo4->Init_KMeans();
    algo5->Init_KMeans();
    algo6->Init_KMeans();
    algo7->Init_KMeans();
    algo8->Init_KMeans();
    
    mod1->Info();
    mod2->Info();
    mod3->Info();
    mod4->Info();    
    mod5->Info();
    mod6->Info();
    mod7->Info();
    mod8->Info();
   
    for (uint iter=0;iter<30;iter++){
			cout << "*****iter=" << iter << endl;
      algo1->Run(1);
      algo2->Run(1);
      algo3->Run(1);
      algo4->Run(1);
      algo5->Run(1);
      algo6->Run(1);
      algo7->Run(1);
      algo8->Run(1);
    }
	    
    mod1->Info();
    mod2->Info();
    mod3->Info();
    mod4->Info();    
    mod5->Info();
    mod6->Info();
    mod7->Info();
    mod8->Info();
		
  }
}
